# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/API/ps.ipynb.

# %% auto 0
__all__ = ['amp_disp']

# %% ../nbs/API/ps.ipynb 4
import cupy as cp
import numpy as np
from typing import Union
import math
import numba

# %% ../nbs/API/ps.ipynb 5
@numba.jit(nopython=True, cache=True,parallel=True)
def _amp_disp_numba(rslc):
    epsilon = np.float32(1e-30)
    nlines, width = rslc.shape[:-1]
    amp_disp = np.empty((nlines,width),dtype=np.float32)
    for j in numba.prange(nlines):
        for i in range(width):
            amp = np.abs(rslc[j,i,:])
            mean = np.mean(amp)+epsilon
            std = np.std(amp)
            amp_disp[j,i] = std/mean
    return amp_disp

# %% ../nbs/API/ps.ipynb 6
_amp_disp_kernel = cp.ElementwiseKernel(
    'raw T rmli_stack, int32 nlines, int32 width, int32 nimages',
    'raw T amp_disp_stack',
    '''
    int k;
    float mean = 0;
    float f_nimages = nimages;
    for (k=0;k<nimages;k++) {
        mean += rmli_stack[i*nimages+k];
    }
    mean += 1e-30;
    mean /= f_nimages;
    float std = 0;
    for (k=0;k<nimages;k++) {
        std += powf(rmli_stack[i*nimages+k]-mean,2);
    }
    std = sqrt(std/f_nimages);
    amp_disp_stack[i] = std/mean;
    ''',
    name = 'amp_disp_kernel',no_return=True)

# %% ../nbs/API/ps.ipynb 7
def _amp_disp_cp(rslc):
    rmli = cp.abs(rslc)
    nlines,width,nimages = rmli.shape
    amp_disp = cp.empty((nlines,width),dtype=cp.float32)
    _amp_disp_kernel(rmli,cp.int32(nlines),cp.int32(width),cp.int32(nimages),
                     amp_disp,size=nlines*width,block_size=128)
    return amp_disp

# %% ../nbs/API/ps.ipynb 8
def amp_disp(rslc:Union[cp.ndarray,np.ndarray], # rslc stack
            )-> Union[cp.ndarray,np.ndarray]: # dispersion index
    '''calculation the amplitude dispersion index from SLC stack.'''
    xp = cp.get_array_module(rslc)
    if xp is np:
        return _amp_disp_numba(rslc)
    elif xp is cp:
        return _amp_disp_cp(rslc)
    else:
        raise TypeError("Only numpy array or cupy array are supported.")
